var CodeMirror = require('codemirror/lib/codemirror')

// modes
require('codemirror/mode/css/css.js')
require('codemirror/mode/htmlmixed/htmlmixed.js')

// addons
require('codemirror/addon/selection/active-line.js')
require('codemirror/addon/edit/matchbrackets.js')

// styles
require('!style-loader!css-loader!codemirror/lib/codemirror.css')

// themes
require('!style-loader!css-loader!codemirror/theme/eclipse.css')

// more styles
require('!style-loader!css-loader!./styles.css')

var DEFAULT_MODE = 'htmlmixed'
var DEFAULT_THEME = 'eclipse'
var DEFAULT_TAB_SIZE = 2

function config(opts) {
  opts || (opts = {})
  has(opts, 'lineNumbers') || (opts.lineNumbers = true)
  has(opts, 'tabSize') || (opts.tabSize = DEFAULT_TAB_SIZE)
  has(opts, 'styleActiveLine') || (opts.styleActiveLine = true)
  has(opts, 'matchBrackets') || (opts.matchBrackets = true)
  has(opts, 'mode') || (opts.mode = DEFAULT_MODE)
  has(opts, 'theme') || (opts.theme = DEFAULT_THEME)
  return opts
}

function enforceMaxLength(max_length) {
  return function(cm, change) {
    if (!change.update)
      return true

    var str = change.text.join("\n")
    var delta = str.length - (cm.indexFromPos(change.to) - cm.indexFromPos(change.from))

    if (delta <= 0)
      return true

    delta = cm.getValue().length + delta - max_length

    if (delta > 0) {
      str = str.substr(0, str.length - delta)
      change.update(change.from, change.to, str.split("\n"))
    }

    return true
  }
}

function self(el, opts) {
  opts || (opts = {})

  var editor = CodeMirror.fromTextArea(el, config(opts))
  editor.on('blur', function() { editor.save() })

  if (opts.max_length)
    editor.on('beforeChange', enforceMaxLength(opts.max_length))

  setTimeout(function() { editor.refresh() }, 100)

  return editor
}

function has(obj, key) {
  return Object.keys(obj || {}).indexOf(key) >= 0
}

self.api = CodeMirror
module.exports = self
